package com.ruoyi.web.controller.api;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.RequestParamsUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.domain.req.*;
import com.ruoyi.system.domain.resp.TRecordResp;
import com.ruoyi.system.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import sun.misc.BASE64Decoder;

import javax.annotation.Resource;
import java.lang.reflect.Field;
import java.util.*;

/**
 * @author zhangcl
 * @date 2022/8/5 17:42
 */
@Api("app接口")
@RestController
@RequestMapping("/api/app")
public class AppController extends BaseController
{
    @Autowired
    private ITRecordService tRecordService;

    @Autowired
    private ITHospitalService tHospitalService;

    @Resource
    private ITDoctorService tDoctorService;

    @Resource
    private ITPatientService tPatientService;

    @Resource
    private ITPatientHospitalDoctorService itPatientHospitalDoctorService;

    @Resource
    private ISysConfigService configService;

    @Resource
    private ITImageService imageService;

    @Value("${aiPost.ip}")
    private String ip;
    @Value("${aiPost.port}")
    private String port;
    @Value("${aiPost.url}")
    private String url;

    /**
     * 完成AI诊断
     */
    @ApiOperation("完成AI诊断")
    @Log(title = "App诊断记录状态", businessType = BusinessType.UPDATE)
    @GetMapping("/ai/finish/{id}")
    public AjaxResult finish(@PathVariable("id") Long id)
    {
        TRecord record = tRecordService.selectTRecordById(id);
        if (record != null)
        {
            record.setStatus("1");
        }
        return toAjax(tRecordService.updateTRecord(record));

    }

    /**
     * 医院列表
     */
    @ApiOperation("医院列表")
    @GetMapping("/hospital/list")
    public AjaxResult hospitalList()
    {
        List<THospital> tHospitals = tHospitalService.selectTHospitalList(new THospital());
        return AjaxResult.success(tHospitals);
    }


    /**
     * 医生信息获取
     *
     * @param doctorId 医生id
     * @return {@code AjaxResult}
     */
    @ApiOperation("医生信息获取")
    @GetMapping("/doctor/info")
    public AjaxResult doctorInformation(Long doctorId)
    {
        return AjaxResult.success(tDoctorService.selectAppDoctorDetailById(doctorId));
    }

    /**
     * 患者预约
     *
     * @param appPatientHospitalDoctor 应用病人医院医生
     * @return {@code AjaxResult}
     */
    @ApiOperation("小程序患者预约")
    @Log(title = "小程序患者预约", businessType = BusinessType.INSERT)
    @GetMapping("/patient/makeAnAppointment")
    public AjaxResult patientsToMakeAnAppointment(AppPatientHospitalDoctor appPatientHospitalDoctor)
    {
        // 患者预约，新增t_record中的一条记录;判断关联表是否存在当前关系
        TPatientHospitalDoctor tPatientHospitalDoctor = new TPatientHospitalDoctor();
        BeanUtils.copyProperties(appPatientHospitalDoctor, tPatientHospitalDoctor);
        tPatientHospitalDoctor.setPatientId(getUserId());
        TRecord tRecord = new TRecord();
        tRecord.setHospitalId(tPatientHospitalDoctor.getHospitalId());
        tRecord.setPatientId(tPatientHospitalDoctor.getPatientId());
        tRecord.setDoctorId(tPatientHospitalDoctor.getDoctorId());
        tRecord.setStatus("0");
        tRecord.setCreateBy(getUsername());
        tRecordService.insertTRecord(tRecord);
        List<TPatientHospitalDoctor> tPHDs = itPatientHospitalDoctorService.selectTPatientHospitalDoctorList(tPatientHospitalDoctor);
        if (tPHDs.isEmpty())
        {
            itPatientHospitalDoctorService.insertTPatientHospitalDoctor(tPatientHospitalDoctor);
        }
        return AjaxResult.success("预约成功");
    }

    @PutMapping("/record/ai/finish/{id}")
    @ApiOperation("患者记录预约到已诊状态转换")
    @Log(title = "患者记录预约到已诊状态转换", businessType = BusinessType.UPDATE)
    public AjaxResult updateStatus(@PathVariable("id") Long id)
    {
        if (id == null)
        {
            return AjaxResult.error("未获取到记录id");
        }
        TRecord tRecord = new TRecord();
        tRecord.setId(id);
        tRecord.setStatus("1");
        tRecordService.updateTRecord(tRecord);
        return AjaxResult.success("更新状态成功", null);
    }

    @GetMapping(value = "/config/ai/refreshtime")
    //@Log(title = "获取AI结果刷新时间接口")
    @ApiOperation("获取AI结果刷新时间接口")
    public AjaxResult getConfigKey()
    {
        String configKey = "t.ai.refreshtime";
        return AjaxResult.success("获取成功" ,configService.selectConfigByKey(configKey));
    }

    /**
     * 诊断记录上传
     *
     * @return {@code AjaxResult}
     */
    @ApiOperation("诊断记录上传")
    @Log(title = "App诊断记录上传", businessType = BusinessType.UPDATE)
    @PostMapping("/record/diagnostic/upload")
    public AjaxResult diagnosticRecordsUploaded(@RequestBody AppRecordReq appRecordReq)
    {
        TRecord tRecord = new TRecord();
        BeanUtils.copyProperties(appRecordReq, tRecord);
        return AjaxResult.success(tRecordService.updateTRecord(tRecord));
    }

    /**
     * 添加病人
     *
     * @param appPatientReq App端添加患者入参实体
     * @return {@code AjaxResult}
     */
    @ApiOperation("添加患者")
    @Log(title = "App患者添加", businessType = BusinessType.INSERT)
    @PostMapping("/patient/add")
    public AjaxResult addPatient(@RequestBody AppPatientReq appPatientReq)
    {
        TPatient tPatient = new TPatient();
        BeanUtils.copyProperties(appPatientReq, tPatient);
        if (UserConstants.NOT_UNIQUE.equals(tPatientService.checkPhoneUnique(tPatient.getPhone())))
        {
            return AjaxResult.error("手机号已注册");
        }
        tPatientService.insertTPatient(tPatient);
        Long identifier = 10000000L + tPatient.getId();
        tPatient.setIdentifier(String.valueOf(identifier));
        tPatientService.updateTPatient(tPatient);
        TPatientHospitalDoctor tPatientHospitalDoctor = new TPatientHospitalDoctor();
        tPatientHospitalDoctor.setHospitalId(appPatientReq.getHospitalId());
        tPatientHospitalDoctor.setDoctorId(appPatientReq.getDoctorId());
        tPatientHospitalDoctor.setPatientId(tPatient.getId());
        List<TPatientHospitalDoctor> tPatientHospitalDoctors = itPatientHospitalDoctorService.selectTPatientHospitalDoctorList(tPatientHospitalDoctor);
        if (tPatientHospitalDoctors.isEmpty())
        {
            itPatientHospitalDoctorService.insertTPatientHospitalDoctor(tPatientHospitalDoctor);
        }
        return AjaxResult.success();
    }

    /**
     * 修改患者
     *
     * @param appPatientReq App端添加患者入参实体
     * @return {@code AjaxResult}
     */
    @ApiOperation("患者信息修改")
    @Log(title = "App患者修改", businessType = BusinessType.UPDATE)
    @PostMapping("/patient/modify")
    public AjaxResult patientsWithModified(@RequestBody AppPatientReq appPatientReq)
    {
        TPatient tPatient = new TPatient();
        BeanUtils.copyProperties(appPatientReq, tPatient);
        return AjaxResult.success(tPatientService.updateTPatient(tPatient));
    }

    @ApiOperation("获取患者详细信息")
    @GetMapping("/patient/info/{id}")
    public AjaxResult getPatientInfo2(@PathVariable("id") Long id)
    {

        return AjaxResult.success(tPatientService.selectTPatientById(id));
    }


    /**
     * 诊断记录列表
     *
     * @param patientId 应用病人医院医生
     * @return {@code AjaxResult}
     */
    @ApiOperation("患者诊断记录历史列表")
    @GetMapping("/record/list/{patientId}")
    public AjaxResult diagnosisOfRecordList(@PathVariable("patientId") Long patientId)
    {
        TPatientHospitalDoctor tPatientHospitalDoctor = new TPatientHospitalDoctor();
        tPatientHospitalDoctor.setPatientId(patientId);
        List<TRecordResp> tRecords = tRecordService.queryRecordListByPHDids(tPatientHospitalDoctor);
        return AjaxResult.success(tRecords);
    }

    /**
     * 患者诊断记录信息细节
     *
     * @param id id
     * @return {@code AjaxResult}
     */
    @ApiOperation("患者诊断记录信息详情")
    @GetMapping("/record/info/{id}")
    public AjaxResult patientsWithDiagnosisOfRecordInformationDetails(@PathVariable("id") Long id)
    {
        return AjaxResult.success(tRecordService.selectById(id));
    }

    /**
     * 患者已诊列表
     *
     * @param recordResultReq 记录结果要求
     * @return {@code AjaxResult}
     */
    @ApiOperation("患者已诊列表")
    @GetMapping("/record/diagnosedList")
    public TableDataInfo patientsDiagnosedList(TRecordResultReq recordResultReq)
    {
        recordResultReq.setStatus("1");
        startPage();
        return getDataTable(tRecordService.selectTRecordListWithoutResult(recordResultReq));
    }

    /**
     * 病人等待名单
     *
     * @param recordResultReq 记录结果要求
     * @return {@code AjaxResult}
     */
    @ApiOperation("患者候诊列表")
    @GetMapping("/record/waitingList")
    public TableDataInfo patientsWaitingList(TRecordResultReq recordResultReq)
    {
        recordResultReq.setStatus("0");
        startPage();
        return getDataTable(tRecordService.selectTRecordListWithoutResult(recordResultReq));
    }

    @ApiOperation("小程序获取患者详细信息")
    @GetMapping("/patient/info")
    public AjaxResult getPatientInfo()
    {
        return AjaxResult.success(tPatientService.selectTPatientById(getUserId()));
    }

    @GetMapping("/patient/record/historyData")
    @ApiOperation("查询患者历史数据图表")
    public AjaxResult getHistoryRecordList(AppRecordReq appRecordReq)
    {
        TRecord tRecord = new TRecord();
        BeanUtils.copyProperties(appRecordReq, tRecord);
        return tRecordService.getHistoryList(tRecord);
    }

    @PostMapping("/ai/{recordId}")
    @ApiOperation("ai接口")
    public AjaxResult aiPost(@PathVariable("recordId") Long recordId, @RequestBody AiPostReq aiPostReq)
    {
        saveImage(aiPostReq,recordId);
        ObjectMapper mapper = new ObjectMapper();
        String URL = RequestParamsUtil.HTTP + ip + RequestParamsUtil.SEPARATOR + port + url;
        JSONObject jsonObject = new JSONObject();
        String result;
        try
        {
            result = HttpUtils.sendPost(URL, mapper.writeValueAsString(aiPostReq));
        }
        catch (Exception e)
        {
            throw new ServiceException(e.getMessage());
        }
        if (!StringUtils.isEmpty(result))
        {

            jsonObject = removeBase64Image(JSONObject.parseObject(result),recordId);
            TRecord tRecord = new TRecord();
            tRecord.setId(recordId);
            tRecord.setStatus("1");
            tRecord.setAiResult(JSONObject.toJSONString(jsonObject, SerializerFeature.WriteMapNullValue));
            tRecord.setAiResult2(JSONObject.toJSONString(jsonObject, SerializerFeature.WriteMapNullValue));
            tRecordService.updateTRecord(tRecord);
        }
        return AjaxResult.success(jsonObject);
    }

    @GetMapping("/image/{recordId}")
    @ApiOperation("获取图片接口")
    public AjaxResult getAiImage(@PathVariable("recordId") Long recordId){
        if(recordId == null){
            return AjaxResult.error("未获取到参数");
        }
        TImage tImage = new TImage();
        tImage.setThirdId(recordId);
        List<TImage> images = imageService.selectTImageList(tImage);
        return AjaxResult.success(images);
    }

    private void saveImage(AiPostReq aiPostReq,Long recordId)
    {
        Base64ResizedRmgSm base64_resized_img_sm = aiPostReq.getBase64_resized_img_sm();
        Base64ResizedRmgSx base64_resized_img_sx = aiPostReq.getBase64_resized_img_sx();
        Map<String, String> smMap = imageKey(base64_resized_img_sm,"base64_resized_img_sm");
//        Map<String, String> sxMap = imageKey(base64_resized_img_sx, "base64_resized_img_sx");
//        smMap.putAll(sxMap);
//        sxMap.forEach((name,value)->{
//            TImage tImage = new TImage();
//            tImage.setThirdId(recordId);
//            tImage.setCreateTime(new Date());
//            tImage.setName(name);
//            String path=GenerateImage(value);
//            tImage.setPath(path);
//            imageService.insertTImage(tImage);
//        });
    }

    public Map<String,String> imageKey(Base64ResizedRmgSm param,String key){
        Map<String,String> map = new HashMap<>();
        if(!StringUtils.isEmpty(param.getFront())){
            map.put(key+"-front",param.getFront());
        }
//        if(!StringUtils.isEmpty(param.getLeft())){
//            map.put(key+"-left",param.getLeft());
//        }
//        if(!StringUtils.isEmpty(param.getFront())){
//            map.put(key+"-right",param.getRight());
//        }
//        if(!CollectionUtils.isEmpty(param.getOther())){
//            for (int i = 0; i < param.getOther().size(); i++)
//            {
//                map.put(key+"-other"+i,param.getOther().get(i));
//            }
//        }
        return map;

    }

    public JSONObject removeBase64Image(JSONObject jsonObject,Long recordId)
    {
        String []keys={"img_sz","img_sz_yudian","img_sz_yuban","img_sz_liewen","img_st","img_sx","img_sx_vein"};
        List<JSONObject> parents=new ArrayList<>();
        try
        {
            parents.add(jsonObject.getJSONObject("data").getJSONObject("shezhi").getJSONObject("shemian"));
            parents.add(jsonObject.getJSONObject("data").getJSONObject("shezhi").getJSONObject("yudian"));
            parents.add(jsonObject.getJSONObject("data").getJSONObject("shezhi").getJSONObject("yuban"));
            parents.add(jsonObject.getJSONObject("data").getJSONObject("shezhi").getJSONObject("liewen"));
            parents.add(jsonObject.getJSONObject("data").getJSONObject("shetai"));
            parents.add(jsonObject.getJSONObject("data").getJSONObject("shexia"));
            parents.add(jsonObject.getJSONObject("data").getJSONObject("shexia"));

            for (int i=0;i<keys.length;i++)
            {
                String imageKey=keys[i];
                String imgStr=parents.get(i).getJSONObject(imageKey).getString("v");
                String path=GenerateImage(imgStr);
                parents.get(i).getJSONObject(keys[i]).put("v",null);
//                TImage tImage = new TImage();
//                tImage.setName(imageKey);
//                tImage.setPath(path);
//                tImage.setCreateTime(new Date());
//                tImage.setThirdId(recordId);
//                imageService.insertTImage(tImage);
                //保存图片到数据库,thirdId(recordId),path,name(imageKey)

            }

        }
        catch (Exception e)
        {
            throw new ServiceException("JSON解析错误:" + jsonObject.toJSONString());
        }
        return jsonObject;
    }


    public static String GenerateImage(String imgStr)
    {

        BASE64Decoder decoder = new BASE64Decoder();
        try
        {
            //Base64解码
            byte[] b = decoder.decodeBuffer(imgStr);
            for (int i = 0; i < b.length; ++i)
            {
                if (b[i] < 0)
                {
                    b[i] += 256;
                }
            }
            return FileUtils.writeImportBytes(b);

        }
        catch (Exception e)
        {
            throw new ServiceException("Base64转图片错误");
        }
    }
}
